#include "hbclass.ch"
#include "common.ch"
#include "fileio.ch"

#define CR_LF    (CHR(13)+CHR(10))

REQUEST DBFCDX

MEMVAR server, get, post, cookie, session

STATIC s_aMap

FIELD USER, NAME, PASSWORD, CODE, PRICE, TOTAL, TITLE

FUNC main()
LOCAL oServer

  IF HB_ARGCHECK("help")
    ? "Usage: app [options]"
    ? "Options:"
    ? "  //help               Print help"
    ? "  //stop               Stop running server"
    RETURN 0
  ENDIF

  IF HB_ARGCHECK("stop")
    HB_MEMOWRIT(".uhttpd.stop", "")
    RETURN 0
  ELSE
    FERASE(".uhttpd.stop")
  ENDIF

  RDDSETDEFAULT("DBFCDX")
  SET(4, "yyyy-mm-dd")

  IF ! HB_FILEEXISTS("users.dbf")
    FERASE("users.cdx")
    DBCREATE("users", {{"USER", "C", 16, 0}, {"PASSWORD", "C", 16, 0}, {"NAME", "C", 50, 0}},, .T., "user")
    OrdCreate("users", "user", "USER")
    DBCLOSEAREA()
  ELSEIF ! HB_FILEEXISTS("users.cdx")
    DBUSEAREA(.T.,, "users",, .F., .F.)
    OrdCreate("users", "user", "USER")
    DBCLOSEAREA()
  ENDIF

  IF ! HB_FILEEXISTS("carts.dbf")
    FERASE("carts.cdx")
    DBCREATE("carts", {{"USER", "C", 16, 0}, {"CODE", "C", 16, 0}, {"AMOUNT", "N", 6, 0}, {"TOTAL", "N", 9, 2}},, .T., "cart")
    OrdCreate("carts", "user", "USER+CODE")
    DBCLOSEAREA()
  ELSEIF ! HB_FILEEXISTS("carts.cdx")
    DBUSEAREA(.T.,, "carts",, .F., .F.)
    OrdCreate("carts", "user", "USER+CODE")
    DBCLOSEAREA()
  ENDIF

  IF ! HB_FILEEXISTS("items.dbf")
    FERASE("items.cdx")
    DBCREATE("items", {{"CODE", "C", 16, 0}, {"TITLE", "C", 80, 0}, {"PRICE", "N", 9, 2}},, .T., "items")
    OrdCreate("items", "code", "CODE")
    DBCLOSEAREA()
  ELSEIF ! HB_FILEEXISTS("item.cdx")
    DBUSEAREA(.T.,, "items",, .F., .F.)
    OrdCreate("items", "code", "CODE")
    DBCLOSEAREA()
  ENDIF

  oServer := UHttpdNew()

  oServer:nPort := 8001
  oServer:bIdle := {|o| IIF(HB_FILEEXISTS(".uhttpd.stop"), (FERASE(".uhttpd.stop"), o:Stop()), NIL)}

  oServer:aMount := ;
     {"/info"             => {|| UProcInfo()}, ;
      "/files/*"          => {|x| QOUT(HB_DIRBASE() + "/files/" + X), UProcFiles(HB_DIRBASE() + "/files/" + X, .F.)}, ;
      "/app/login"        => @proc_login(), ;
      "/app/logout"       => @proc_logout(), ;
      "/app/account"      => @proc_account(), ;
      "/app/account/edit" => @proc_account_edit(), ;
      "/app/register"     => @proc_register(), ;
      "/app/main"         => @proc_main(), ;
      "/app/shopping"     => @proc_shopping(), ;
      "/app/cart"         => @proc_cart(), ;
      "/"                 => {|| URedirect("/app/login")}}

  IF ! oServer:Run()
    ? "Server error:", oServer:cError
    RETURN 1
  ENDIF

RETURN 0


STATIC FUNC proc_login()
LOCAL cUser

  IF server["REQUEST_METHOD"] == "POST"
    DBUSEAREA(.T.,, "users", "users", .T., .T.)
    OrdSetFocus("user")
    cUser := PADR(HB_HGetDef(post, "user", ""), 16)
    USessionStart()
    IF !EMPTY(cUser) .AND. DBSEEK(cUser, .F.) .AND. ! DELETED() .AND. ;
       PADR(HB_HGetDef(post, "password", ""), 16) == FIELD->PASSWORD
      session["user"] := cUser
      URedirect("main")
    ELSE
      URedirect("login?err")
      USessionDestroy()
    ENDIF
    DBCLOSEAREA()
  ELSE
    IF HB_HHasKey(get, "err")
      RETURN {"errtext"=>"Invalid user name or password!"}
    ENDIF
    RETURN {=>}
  ENDIF
RETURN NIL


STATIC FUNC proc_logout()
  USessionDestroy() 
RETURN {=>}


STATIC FUNC proc_main()
  USessionStart()
  IF ! HB_HHasKey(session, "user")
    URedirect("/app/login")
    RETURN NIL
  ENDIF
RETURN {=>}


STATIC FUNC proc_shopping()
LOCAL oW, nT, cCode

  USessionStart()
  IF ! HB_HHasKey(session, "user");  URedirect("/app/login");  RETURN NIL
  ENDIF

  DBUSEAREA(.T.,, "carts", "carts", .T., .F.)
  OrdSetFocus("user")
  DBUSEAREA(.T.,, "items", "items", .T., .T.)
  OrdSetFocus("code")

  IF HB_HHasKey(get, "add")
    cCode := PADR(get["add"], 16)
    IF items->(DBSEEK(cCode)) .AND. carts->(FLOCK())
      IF ! carts->(DBSEEK(session["user"] + cCode))
        carts->(DBAPPEND())
        carts->USER := session["user"]
        carts->CODE := cCode
      ENDIF
      carts->AMOUNT += 1
      carts->TOTAL += items->PRICE
      carts->(DBUNLOCK())
    ENDIF
    URedirect("shopping")
    RETURN NIL
  ENDIF

  DBSELECTAREA("carts")
  ORDSCOPE(0, session["user"])
  ORDSCOPE(1, session["user"])
  nT := 0
  carts->(DBEVAL({|| nT += TOTAL}))
  DBSELECTAREA("items")
  oW := UWBrowseNew("br_item")
  oW:AddColumn(101, "Item No.",    "CODE")
  oW:AddColumn(102, "Title",       "TITLE")
  oW:AddColumn(103, "Price",       "PRICE")
  oW:AddColumn(104, "",            {|| ULink("Add to cart", "?add=" + TRIM(CODE))}, .T.)
  oW:nPageSize := 10
  IF HB_HHasKey(get, "_pos")
    oW:nPos := VAL(get["_pos"])
  ENDIF
RETURN {"browse"=>oW:Output(), "cartsum"=>nT}


STATIC FUNC proc_cart()
LOCAL oW, nT, cCode

  USessionStart()
  IF ! HB_HHasKey(session, "user");  URedirect("/app/login");  RETURN NIL
  ENDIF

  DBUSEAREA(.T.,, "items", "items", .T., .T.)
  OrdSetFocus("code")
  DBUSEAREA(.T.,, "carts", "carts", .T., .F.)
  OrdSetFocus("user")

  IF HB_HHasKey(get, "del")
    cCode := PADR(get["del"], 16)
    IF items->(DBSEEK(cCode)) .AND. carts->(FLOCK())
      IF carts->(DBSEEK(session["user"] + cCode))
        carts->(DBDELETE())
        carts->USER := ""
        carts->CODE := cCode
      ENDIF
      carts->(DBUNLOCK())
    ENDIF
    URedirect("cart")
    RETURN NIL
  ENDIF

  ORDSCOPE(0, session["user"])
  ORDSCOPE(1, session["user"])
  nT := 0
  carts->(DBEVAL({|| nT += TOTAL}))

  oW := UWBrowseNew("br_cart")
  oW:AddColumn(101, "Item No.",    "CODE")
  oW:AddColumn(102, "Title",       {|| items->(DBSEEK(carts->CODE, .F.), TITLE)})
  oW:AddColumn(103, "Amount",      "AMOUNT")
  oW:AddColumn(104, "Total",       "TOTAL")
  oW:AddColumn(104, "",            {|| ULink("Delete", "?del=" + TRIM(CODE))}, .T.)
  oW:nPageSize := 10
  IF HB_HHasKey(get, "_pos")
    oW:nPos := VAL(get["_pos"])
  ENDIF
RETURN {"browse"=>oW:Output(), "cartsum"=>nT}


STATIC FUNC proc_account()
  USessionStart()
  IF ! HB_HHasKey(session, "user");  URedirect("/app/login");  RETURN NIL
  ENDIF
  DBUSEAREA(.T.,, "users", "users", .T., .F.)
  OrdSetFocus("user")
  DBSEEK(session["user"], .F.)
RETURN {"user"=>users->USER, "name"=>users->NAME}


STATIC FUNC proc_account_edit()
LOCAL cName, cPassword1, cPassword2, aRet

  USessionStart()
  IF ! HB_HHasKey(session, "user");  URedirect("/app/login");  RETURN NIL
  ENDIF
  DBUSEAREA(.T.,, "users", "users", .T., .F.)
  OrdSetFocus("user")
  DBSEEK(session["user"], .F.)

  cName := users->NAME
  IF HB_HHasKey(session, "formdata_account/edit")
    cName := session["formdata_account/edit", "name"]
  ENDIF
  IF server["REQUEST_METHOD"] == "POST"
    cName := HB_HGetDef(post, "name", "")
    cPassword1 := HB_HGetDef(post, "password1", "")
    cPassword2 := HB_HGetDef(post, "password2", "")
    IF EMPTY(cName)
      session["formdata_account/edit"] := {"name"=>cName}
      URedirect("?err=1")
    ELSEIF (! EMPTY(cPassword1) .OR. ! EMPTY(cPassword2)) .AND. ! (cPassword1 == cPassword2)
      session["formdata_account/edit"] := {"name"=>cName}
      URedirect("?err=2")
    ELSE
      FLOCK()
      NAME := cName
      IF ! EMPTY(cPassword1)
         PASSWORD := cPassword1
      ENDIF
      DBUNLOCK()
      IF HB_HHasKey(session, "formdata_account/edit")
        HB_HDel(session, "formdata_account/edit")
      ENDIF
      URedirect("/app/account")
    ENDIF
    RETURN NIL
  ENDIF

  aRet := {"user"=>users->USER, "name"=>cName}
  IF HB_HHasKey(get, "err")
    IF get["err"] == "1"
      aRet["errtext"] := "Name value should not be empty!"
    ELSEIF get["err"] == "2"
      aRet["errtext"] := "Passwords do not match!"
    ENDIF
  ENDIF
RETURN aRet


STATIC FUNC proc_register()
LOCAL cUser, cName, cPassword1, cPassword2, aRet

  USessionStart()
  cUser := ""
  cName := ""
  IF HB_HHasKey(session, "formdata_register")
    cUser := session["formdata_register", "user"]
    cName := session["formdata_register", "name"]
  ENDIF
  IF server["REQUEST_METHOD"] == "POST"
    DBUSEAREA(.T.,, "users", "users", .T., .F.)
    OrdSetFocus("user")
    cUser := HB_HGetDef(post, "user", "")
    cName := HB_HGetDef(post, "name", "")
    cPassword1 := HB_HGetDef(post, "password1", "")
    cPassword2 := HB_HGetDef(post, "password2", "")

    IF EMPTY(cUser) .OR. EMPTY(cName) .OR. EMPTY(cPassword1) .OR. EMPTY(cPassword2)
      session["formdata_register"] := {"user"=>cUser, "name"=>cName}
      URedirect("?err=1")
    ELSEIF !(cPassword1 == cPassword2)
      session["formdata_register"] := {"user"=>cUser, "name"=>cName}
      URedirect("?err=2")
    ELSEIF DBSEEK(cUser, .F.)
      session["formdata_register"] := {"user"=>cUser, "name"=>cName}
      URedirect("?err=3")
    ELSE
      FLOCK()
      DBAPPEND()
      USER := cUser
      NAME := cName
      PASSWORD := cPassword1
      DBUNLOCK()
      USessionDestroy()
      USessionStart()
      session["user"] := cUser
      URedirect("/app/main")
    ENDIF
    RETURN NIL
  ENDIF
  aRet := {"user"=>cUser, "name"=>cName}
  IF HB_HHasKey(get, "err")
    IF get["err"] == "1"
      aRet["errtext"] := "All fields are required!"
    ELSEIF get["err"] == "2"
      aRet["errtext"]  := "Passwords does not match!"
    ELSEIF get["err"] == "3"
      aRet["errtext"]  := "This user already exists!"
    ENDIF
  ENDIF
RETURN aRet
